@@ -1,8 +1,8 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
-# $Id: portarchivefetch.tcl 80587 2011-07-15 14:09:49Z jmr@macports.org $
+# $Id: portarchivefetch.tcl 140689 2015-09-30 06:27:47Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Inc.
-# Copyright (c) 2004 - 2011 The MacPorts Project
+# Copyright (c) 2004 - 2013 The MacPorts Project
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@ package require portutil 1.0
 package require Pextlib 1.0
 
 set org.macports.archivefetch [target_new org.macports.archivefetch portarchivefetch::archivefetch_main]
-target_init ${org.macports.archivefetch} portarchivefetch::archivefetch_init
+#target_init ${org.macports.archivefetch} portarchivefetch::archivefetch_init
 target_provides ${org.macports.archivefetch} archivefetch
 target_requires ${org.macports.archivefetch} main
 target_prerun ${org.macports.archivefetch} portarchivefetch::archivefetch_start
@@ -65,19 +65,47 @@ default archive_sites.listpath {"port1.0/fetch"}
 default archive.subdir {${subport}}
 
 proc portarchivefetch::filter_sites {} {
-    global prefix porturl
+    global prefix frameworks_dir applications_dir porturl \
+        portfetch::mirror_sites::sites portfetch::mirror_sites::archive_type \
+        portfetch::mirror_sites::archive_prefix \
+        portfetch::mirror_sites::archive_frameworks_dir \
+        portfetch::mirror_sites::archive_applications_dir
+
+    # get defaults from ports tree resources
     set mirrorfile [get_full_archive_sites_path]
     if {[file exists $mirrorfile]} {
         source $mirrorfile
     }
+    # get archive_sites.conf values
+    foreach {key val} [get_archive_sites_conf_values] {
+        set $key $val
+    }
+
     set ret {}
     foreach site [array names portfetch::mirror_sites::archive_prefix] {
-        if {$portfetch::mirror_sites::archive_prefix($site) == $prefix} {
-            lappend ret $site
+        set missing 0
+        foreach var {archive_frameworks_dir archive_applications_dir archive_type} {
+            if {![info exists portfetch::mirror_sites::${var}($site)]} {
+                ui_warn "no $var configured for site '$site'"
+                set missing 1
+            }
+        }
+        if {$missing} {
+            continue
+        }
+        if {$portfetch::mirror_sites::sites($site) ne {} &&
+            $portfetch::mirror_sites::archive_prefix($site) == $prefix &&
+            $portfetch::mirror_sites::archive_frameworks_dir($site) == $frameworks_dir &&
+            $portfetch::mirror_sites::archive_applications_dir($site) == $applications_dir &&
+            ![catch {archiveTypeIsSupported $portfetch::mirror_sites::archive_type($site)}]} {
+            # using the archive type as a tag
+            lappend ret ${site}::$portfetch::mirror_sites::archive_type($site)
         }
     }
-    if {[file rootname [file tail $porturl]] == [file rootname [file tail [get_portimage_path]]]} {
-        lappend ret [string range $porturl 0 end-[string length [file tail $porturl]]]
+
+    # check if porturl itself points to an archive
+    if {[file rootname [file tail $porturl]] eq [file rootname [get_portimage_name]] && [file extension $porturl] ne ""} {
+        lappend ret [string range $porturl 0 end-[string length [file tail $porturl]]]:[string range [file extension $porturl] 1 end]
         archive.subdir
     }
     return $ret
@@ -87,21 +115,22 @@ set_ui_prefix
 
 # Checks possible archive files to assemble url lists for later fetching
 proc portarchivefetch::checkarchivefiles {urls} {
-    global all_archive_files archivefetch.fulldestpath portarchivetype \
-           version revision portvariants archive_sites
+    global all_archive_files archivefetch.fulldestpath archive_sites
     upvar $urls fetch_urls
 
     # Define archive directory path
-    set archive.path [get_portimage_path]
-    set archivefetch.fulldestpath [file dirname ${archive.path}]
+    set archivefetch.fulldestpath [file join [option portdbpath] incoming/verified]
+    set archive.rootname [file rootname [get_portimage_name]]
 
-    # throws an error if unsupported
-    archiveTypeIsSupported $portarchivetype
-
-    set archive.file [file tail ${archive.path}]
-    lappend all_archive_files ${archive.file}
-    if {[info exists archive_sites]} {
-        lappend fetch_urls archive_sites ${archive.file}
+    foreach entry [option archive_sites] {
+        # the archive type is used as a tag
+        set type [lindex [split $entry :] end]
+        if {![info exists seen($type)]} {
+            set archive.file "${archive.rootname}.${type}"
+            lappend all_archive_files ${archive.file}
+            lappend fetch_urls $type ${archive.file}
+            set seen($type) 1
+        }
     }
 }
 
@@ -124,10 +153,10 @@ proc portarchivefetch::checkfiles {urls} {
 # Perform a standard fetch, assembling fetch urls from
 # the listed url variable and associated archive file
 proc portarchivefetch::fetchfiles {args} {
-    global archivefetch.fulldestpath UI_PREFIX
-    global archivefetch.user archivefetch.password archivefetch.use_epsv \
-           archivefetch.ignore_sslcert
-    global portverbose ports_binary_only
+    global archivefetch.fulldestpath UI_PREFIX \
+           archivefetch.user archivefetch.password archivefetch.use_epsv \
+           archivefetch.ignore_sslcert \
+           portverbose ports_binary_only
     variable archivefetch_urls
     variable ::portfetch::urlmap
 
@@ -141,16 +170,6 @@ proc portarchivefetch::fetchfiles {args} {
         }
     }
     set incoming_path [file join [option portdbpath] incoming]
-    if {![file isdirectory $incoming_path]} {
-        if {[catch {file mkdir $incoming_path} result]} {
-            elevateToRoot "archivefetch"
-            set elevated yes
-            if {[catch {file mkdir $incoming_path} result]} {
-                return -code error [format [msgcat::mc "Unable to create archive fetch path: %s"] $result]
-            }
-        }
-    }
-    chownAsRoot ${archivefetch.fulldestpath}
     chownAsRoot $incoming_path
     if {[info exists elevated] && $elevated == yes} {
         dropPrivileges
@@ -167,13 +186,19 @@ proc portarchivefetch::fetchfiles {args} {
     if {${archivefetch.ignore_sslcert} != "no"} {
         lappend fetch_options "--ignore-ssl-cert"
     }
-    if {$portverbose == "yes"} {
-        lappend fetch_options "-v"
+    if {$portverbose eq "yes"} {
+        lappend fetch_options "--progress"
+        lappend fetch_options "builtin"
+    } elseif {[llength [info commands ui_progress_download]] > 0} {
+        lappend fetch_options "--progress"
+        lappend fetch_options "ui_progress_download"
     }
     set sorted no
 
+    set existing_archive [find_portarchive_path]
+
     foreach {url_var archive} $archivefetch_urls {
-        if {![file isfile ${archivefetch.fulldestpath}/${archive}]} {
+        if {![file isfile ${archivefetch.fulldestpath}/${archive}] && $existing_archive eq ""} {
             ui_info "$UI_PREFIX [format [msgcat::mc "%s doesn't seem to exist in %s"] $archive ${archivefetch.fulldestpath}]"
             if {![file writable ${archivefetch.fulldestpath}]} {
                 return -code error [format [msgcat::mc "%s must be writable"] ${archivefetch.fulldestpath}]
@@ -189,9 +214,10 @@ proc portarchivefetch::fetchfiles {args} {
                 ui_error [format [msgcat::mc "No defined site for tag: %s, using archive_sites"] $url_var]
                 set urlmap($url_var) $urlmap(archive_sites)
             }
+            set failed_sites 0
             unset -nocomplain fetched
             foreach site $urlmap($url_var) {
-                if {[string index $site end] != "/"} {
+                if {[string index $site end] ne "/"} {
                     append site "/[option archive.subdir]"
                 } else {
                     append site [option archive.subdir]
@@ -206,6 +232,10 @@ proc portarchivefetch::fetchfiles {args} {
                 } else {
                     ui_debug "[msgcat::mc "Fetching archive failed:"]: $result"
                     file delete -force "${incoming_path}/${archive}.TMP"
+                    incr failed_sites
+                    if {$failed_sites > 2 && ![tbool ports_binary_only] && ![_archive_available]} {
+                        break
+                    }
                 }
             }
             if {[info exists fetched]} {
@@ -228,18 +258,22 @@ proc portarchivefetch::fetchfiles {args} {
                         ui_debug "openssl output: $result"
                     }
                 }
+                file delete -force $signature
                 if {!$verified} {
-                    return -code error "Failed to verify signature for archive!"
-                }
-                if {[catch {file rename -force "${incoming_path}/${archive}.TMP" "${archivefetch.fulldestpath}/${archive}"} result]} {
+                    # fall back to building from source (or error out later if binary only mode)
+                    ui_warn "Failed to verify signature for archive!"
+                    file delete -force "${incoming_path}/${archive}.TMP"
+                    break
+                } elseif {[catch {file rename -force "${incoming_path}/${archive}.TMP" "${archivefetch.fulldestpath}/${archive}"} result]} {
                     ui_debug "$::errorInfo"
                     return -code error "Failed to move downloaded archive into place: $result"
                 }
-                file delete -force $signature
                 set archive_exists 1
+                break
             }
         } else {
             set archive_exists 1
+            break
         }
     }
     if {[info exists archive_exists]} {
@@ -250,7 +284,7 @@ proc portarchivefetch::fetchfiles {args} {
         }
         return 0
     }
-    if {[info exists ports_binary_only] && $ports_binary_only == "yes"} {
+    if {([info exists ports_binary_only] && $ports_binary_only eq "yes") || [_archive_available]} {
         return -code error "archivefetch failed for [option subport] @[option version]_[option revision][option portvariants]"
     } else {
         return 0
@@ -258,24 +292,22 @@ proc portarchivefetch::fetchfiles {args} {
 }
 
 # Initialize archivefetch target and call checkfiles.
-proc portarchivefetch::archivefetch_init {args} {
-    global porturl portarchivetype
-    # installing straight from a binary archive
-    if {[file rootname [file tail $porturl]] == [file rootname [file tail [get_portimage_path]]] && [file extension $porturl] != ""} {
-        set portarchivetype [string range [file extension $porturl] 1 end]
-    }
-    return 0
-}
+#proc portarchivefetch::archivefetch_init {args} {
+#    return 0
+#}
 
 proc portarchivefetch::archivefetch_start {args} {
     variable archivefetch_urls
-    global UI_PREFIX subport all_archive_files ports_source_only
-    if {![tbool ports_source_only]} {
+    global UI_PREFIX subport all_archive_files destroot target_state_fd \
+           ports_source_only ports_binary_only
+    if {![tbool ports_source_only] && ([tbool ports_binary_only] ||
+            !([check_statefile target org.macports.destroot $target_state_fd] && [file isdirectory $destroot]))} {
         portarchivefetch::checkfiles archivefetch_urls
     }
     if {[info exists all_archive_files] && [llength $all_archive_files] > 0} {
         ui_msg "$UI_PREFIX [format [msgcat::mc "Fetching archive for %s"] $subport]"
     }
+    portfetch::check_dns
 }
 
 # Main archive fetch routine
