@@ -1,7 +1,8 @@
 # registry.tcl
 #
+# Copyright (c) 2004-2005, 2007-2012 The MacPorts Project
 # Copyright (c) 2004 Will Barton <wbb4@opendarwin.org>
-# Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2002 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -12,7 +13,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 # 
@@ -45,7 +46,7 @@ namespace eval registry {
 # Begin creating a new registry entry for the port version_revision+variant
 # This process assembles the directory name and creates a receipt dlist
 proc new_entry {name version {revision 0} {variants ""} {epoch 0} } {
-	global macports::registry.path macports::registry.format macports::registry.installtype macports::prefix
+	global macports::registry.path macports::registry.format macports::prefix
 
 	
 	# Make sure we don't already have an entry in the Registry for this
@@ -60,13 +61,9 @@ proc new_entry {name version {revision 0} {variants ""} {epoch 0} } {
 		property_store $ref variants $variants
 		property_store $ref epoch $epoch
 		property_store $ref date [clock seconds]
-		property_store $ref installtype ${macports::registry.installtype}
+		property_store $ref installtype image
 		property_store $ref receipt_f ${macports::registry.format}
-		if { ${macports::registry.installtype} == "image" } {
-			set imagedir [file join ${macports::registry.path} software ${name} ${version}_${revision}${variants}]
-			property_store $ref imagedir $imagedir
-			property_store $ref active 0
-		}
+        property_store $ref active 0
 
 		return $ref
 	} else {
@@ -159,10 +156,10 @@ proc installed {{name ""} {version ""}} {
     }
     
     if { [llength $rlist] < 1 } {
-        if { $name == "" } {
+        if { $name eq "" } {
             return -code error "Registry error: No ports registered as installed."
         } else {
-            if { $version == "" } {
+            if { $version eq "" } {
                 return -code error "Registry error: $name not registered as installed."
             } else {
                 return -code error "Registry error: $name $version not registered as installed."
@@ -199,7 +196,7 @@ proc active {{name ""}} {
     }
 	
 	if { [llength $rlist] < 1 } {
-		if { $name == "" } {
+		if { $name eq "" } {
 			return -code error "Registry error: No ports registered as active."
 		} else {
 			return -code error "Registry error: $name not registered as installed & active."
@@ -323,7 +320,7 @@ proc fileinfo_for_file {fname} {
     # (we won't store the md5 of the target of links since it's meaningless
     # and $statvar(mode) tells us that links are links).
     if {![catch {file lstat $fname statvar}]} {
-	if {[file isfile $fname] && [file type $fname] != "link"} {
+	if {[::file isfile $fname] && [::file type $fname] ne "link"} {
 	    if {[catch {md5 file $fname} md5sum] == 0} {
 		# Create a line that matches md5(1)'s output
 		# for backwards compatibility
@@ -349,8 +346,8 @@ proc fileinfo_for_index {flist} {
 
 	set rval [list]
 	foreach file $flist {
-		if {[string index $file 0] != "/"} {
-			set file [file join $prefix $file]
+		if {[string index $file 0] ne "/"} {
+			set file [::file join $prefix $file]
 		}
 		lappend rval [fileinfo_for_file $file]
 	}
@@ -399,19 +396,19 @@ proc exclusive_lock {} {
     if {$nlocked > 1} {
         return
     }
-    set lockpath [file join ${registry.path} registry .registry.lock]
+    set lockpath [::file join ${registry.path} registry .registry.lock]
     if {![info exists lockfd]} {
-        if {![file writable [file dirname $lockpath]]} {
+        if {![::file writable [::file dirname $lockpath]]} {
             # skip locking, registry can't be modified anyway
             return
         }
         set lockfd [::open $lockpath w]
     }
     if {[catch {flock $lockfd -exclusive -noblock} result]} {
-        if {$result == "EAGAIN"} {
+        if {$result eq "EAGAIN"} {
             ui_msg "Waiting for lock on $lockpath"
             flock $lockfd -exclusive
-        } elseif {$result == "EOPNOTSUPP"} {
+        } elseif {$result eq "EOPNOTSUPP"} {
             # Locking not supported, just return
             ui_debug "flock not supported, not locking registry"
         } else {
@@ -459,31 +456,32 @@ proc convert_to_sqlite {} {
 
         set installtype [receipt_flat::property_retrieve $iref installtype]
         lappend proplist installtype $installtype
-        if { $installtype == "image" } {
-            set imagedir [receipt_flat::property_retrieve $iref imagedir]
-            set contents [receipt_flat::property_retrieve $iref contents]
-            set imagefiles [list]
-            set idlen [string length $imagedir]
-            foreach f $contents {
-                set fullpath [lindex $f 0]
-                # strip image dir from start
+        set location [receipt_flat::property_retrieve $iref location]
+        if {$location == 0} {
+            set location [receipt_flat::property_retrieve $iref imagedir]
+        }
+        set contents [receipt_flat::property_retrieve $iref contents]
+        set imagefiles [list]
+        set idlen [string length $location]
+        foreach f $contents {
+            set fullpath [lindex $f 0]
+            # strip image dir from start
+            if {[string range $fullpath 0 [expr {$idlen - 1}]] == $location} {
                 set path [string range $fullpath $idlen [string length $fullpath]]
-                lappend imagefiles $path
-            }
-            lappend proplist imagefiles $imagefiles
-            set active [receipt_flat::property_retrieve $iref active]
-            if {$active} {
-                set state installed
-                lappend proplist files [receipt_flat::port_registered $iname]
             } else {
-                set state imaged
+                set path $fullpath
             }
-        } else {
-            set imagedir ""
+            lappend imagefiles $path
+        }
+        lappend proplist imagefiles $imagefiles
+        set active [receipt_flat::property_retrieve $iref active]
+        if {$active} {
             set state installed
             lappend proplist files [receipt_flat::port_registered $iname]
+        } else {
+            set state imaged
         }
-        lappend proplist location $imagedir
+        lappend proplist location $location
         lappend proplist state $state
         
         receipt_flat::open_dep_map
